@@ -15,6 +15,7 @@
 
 from __future__ import annotations
 
+import contextlib
 import io
 import logging
 import pydoc
@@ -26,8 +27,7 @@ from inspect import CO_COROUTINE  # pylint: disable=no-name-in-module
 from types import TracebackType
 from typing import Any
 
-import dill as pickle  # type: ignore
-import elasticapm  # type: ignore
+import dill as pickle  # type: ignore[import]  # nosec: B403
 from tornado.web import HTTPError
 
 from .. import EVENT_REDIS, EVENT_SHUTDOWN
@@ -53,14 +53,14 @@ def get_module_info() -> ModuleInfo:
 class PrintWrapper:  # pylint: disable=too-few-public-methods
     """Wrapper for print()."""
 
-    def __init__(self, output: io.TextIOBase) -> None:  # noqa: D107
-        self._output: io.TextIOBase = output
-
-    def __call__(self, *args: Any, **kwargs: Any) -> None:
+    def __call__(self, *args: Any, **kwargs: Any) -> None:  # noqa: D102
         if "file" not in kwargs:
             kwargs["file"] = self._output
         print(*args, **kwargs)
 
+    def __init__(self, output: io.TextIOBase) -> None:  # noqa: D107
+        self._output: io.TextIOBase = output
+
 
 class Backdoor(APIRequestHandler):
     """The request handler for the backdoor API."""
@@ -73,6 +73,91 @@ class Backdoor(APIRequestHandler):
 
     sessions: dict[str, dict[str, Any]] = {}
 
+    async def backup_session(self) -> bool:
+        """Backup a session using Redis and return whether it succeeded."""
+        session_id: None | str = self.request.headers.get("X-Backdoor-Session")
+        if not (EVENT_REDIS.is_set() and session_id in self.sessions):
+            return False
+        session: dict[str, Any] = self.sessions[session_id].copy()
+        session["self"] = None
+        session["app"] = None
+        session["settings"] = None
+        for key, value in tuple(session.items()):
+            try:
+                session[key] = pickle.dumps(value, PICKLE_PROTOCOL)
+            except Exception:  # pylint: disable=broad-except
+                del session[key]
+        return bool(
+            await self.redis.setex(
+                f"{self.redis_prefix}:backdoor-session:{session_id}",
+                60 * 60 * 24 * 7,  # time to live in seconds (1 week)
+                b85encode(pickle.dumps(session, PICKLE_PROTOCOL)),
+            )
+        )
+
+    def finish_pickled_dict(self, **kwargs: Any) -> Future[None]:
+        """Finish with a pickled dictionary."""
+        return self.finish(pickle.dumps(kwargs, self.get_protocol_version()))
+
+    def get_flags(self, flags: int) -> int:
+        """Get compiler flags."""
+        import __future__  # pylint: disable=import-outside-toplevel
+
+        for ftr in self.request.headers.get("X-Future-Feature", "").split(","):
+            if (feature := ftr.strip()) in __future__.all_feature_names:
+                flags |= getattr(__future__, feature).compiler_flag
+
+        return flags
+
+    def get_protocol_version(self) -> int:
+        """Get the protocol version for the response."""
+        try:  # pylint: disable=line-too-long
+            return min(  # type: ignore[no-any-return]
+                int(
+                    self.request.headers.get("X-Pickle-Protocol")  # type: ignore[arg-type]  # noqa: B950
+                ),
+                pickle.HIGHEST_PROTOCOL,
+            )
+        except (TypeError, ValueError):
+            return 5
+
+    async def load_session(self) -> dict[str, Any]:
+        """Load the backup of a session or create a new one."""
+        session_id: None | str = self.request.headers.get("X-Backdoor-Session")
+        if not session_id:
+            session: dict[str, Any] = {
+                "__builtins__": __builtins__,
+                "__name__": "this",
+            }
+        elif session_id in self.sessions:
+            session = self.sessions[session_id]
+        else:
+            session_pickle = (
+                await self.redis.get(
+                    f"{self.redis_prefix}:backdoor-session:{session_id}"
+                )
+                if EVENT_REDIS.is_set()
+                else None
+            )
+            if session_pickle:
+                session = pickle.loads(b85decode(session_pickle))  # nosec: B301
+                for key, value in session.items():
+                    try:
+                        session[key] = pickle.loads(value)  # nosec: B301
+                    except Exception:  # pylint: disable=broad-except
+                        logger.exception("Loading the session failed")
+                        if self.apm_client:
+                            self.apm_client.capture_exception()
+            else:
+                session = {
+                    "__builtins__": __builtins__,
+                    "__name__": "this",
+                }
+                if self.settings.get("TESTING"):
+                    session["session_id"] = session_id
+            self.sessions[session_id] = session
+        return self.update_session(session)
+
     async def post(self, mode: str) -> None:  # noqa: C901
         # pylint: disable=too-complex, too-many-branches, too-many-statements
         """Handle POST requests to the backdoor API."""
@@ -110,8 +195,10 @@ class Backdoor(APIRequestHandler):
                 session["help"] = pydoc.Helper(io.StringIO(), output)
             try:
                 try:
-                    result: Any = eval(  # pylint: disable=eval-used
-                        code, session
+                    result: Any = (
+                        eval(  # pylint: disable=eval-used  # nosec: B307
+                            code, session
+                        )
                     )
                     if code.co_flags & CO_COROUTINE:
                         result = await result
@@ -122,7 +209,7 @@ class Backdoor(APIRequestHandler):
                 new_args = []
                 for arg in exc.args:
                     try:
-                        pickle.dumps(arg, PICKLE_PROTOCOL)
+                        pickle.dumps(arg, self.get_protocol_version())
                     except Exception:  # pylint: disable=broad-except
                         new_args.append(repr(arg))
                     else:
@@ -135,10 +222,8 @@ class Backdoor(APIRequestHandler):
                 )
             except Exception as exc:  # pylint: disable=broad-except
                 exception = exc  # pylint: disable=redefined-variable-type
-                try:
+                with contextlib.suppress(UnboundLocalError):
                     del result
-                except UnboundLocalError:
-                    pass
             else:
                 if result is not None:  # noqa: F821
                     if result is session.get(  # noqa: F821
@@ -168,7 +253,7 @@ class Backdoor(APIRequestHandler):
         try:
             result_tuple = (
                 result_tuple[0] or repr(result_tuple[1]),
-                pickle.dumps(result_tuple[1], PICKLE_PROTOCOL),
+                pickle.dumps(result_tuple[1], self.get_protocol_version()),
             )
         except Exception:  # pylint: disable=broad-except
             result_tuple = (
@@ -183,87 +268,11 @@ class Backdoor(APIRequestHandler):
             else result_tuple,
         )
 
-    def finish_pickled_dict(self, **kwargs: Any) -> Future[None]:
-        """Finish with a pickled dictionary."""
-        return self.finish(pickle.dumps(kwargs, PICKLE_PROTOCOL))
-
-    def get_flags(self, flags: int) -> int:
-        """Get compiler flags."""
-        import __future__  # pylint: disable=import-outside-toplevel
-
-        for ftr in self.request.headers.get("X-Future-Feature", "").split(","):
-            if (feature := ftr.strip()) in __future__.all_feature_names:
-                flags |= getattr(__future__, feature).compiler_flag
-
-        return flags
-
     def update_session(self, session: dict[str, Any]) -> dict[str, Any]:
         """Add request-specific stuff to the session."""
         session.update(self=self, app=self.application, settings=self.settings)
         return session
 
-    async def load_session(self) -> dict[str, Any]:
-        """Load the backup of a session or create a new one."""
-        session_id: None | str = self.request.headers.get("X-Backdoor-Session")
-        if not session_id:
-            session: dict[str, Any] = {
-                "__builtins__": __builtins__,
-                "__name__": "this",
-            }
-        elif session_id in self.sessions:
-            session = self.sessions[session_id]
-        else:
-            session_pickle = (
-                await self.redis.get(
-                    f"{self.redis_prefix}:backdoor-session:{session_id}"
-                )
-                if EVENT_REDIS.is_set()
-                else None
-            )
-            if session_pickle:
-                session = pickle.loads(b85decode(session_pickle))
-                for key, value in session.items():
-                    try:
-                        session[key] = pickle.loads(value)
-                    except Exception as exc:  # pylint: disable=broad-except
-                        logger.exception(exc)
-                        apm: None | elasticapm.Client = self.settings.get(
-                            "ELASTIC_APM_CLIENT"
-                        )
-                        if apm:
-                            apm.capture_exception()
-            else:
-                session = {
-                    "__builtins__": __builtins__,
-                    "__name__": "this",
-                }
-                if self.settings.get("TESTING"):
-                    session["session_id"] = session_id
-            self.sessions[session_id] = session
-        return self.update_session(session)
-
-    async def backup_session(self) -> bool:
-        """Backup a session using Redis and return whether it succeeded."""
-        session_id: None | str = self.request.headers.get("X-Backdoor-Session")
-        if not (EVENT_REDIS.is_set() and session_id in self.sessions):
-            return False
-        session: dict[str, Any] = self.sessions[session_id].copy()
-        session["self"] = None
-        session["app"] = None
-        session["settings"] = None
-        for key, value in tuple(session.items()):
-            try:
-                session[key] = pickle.dumps(value, PICKLE_PROTOCOL)
-            except Exception:  # pylint: disable=broad-except
-                del session[key]
-        return bool(
-            await self.redis.setex(
-                f"{self.redis_prefix}:backdoor-session:{session_id}",
-                60 * 60 * 24 * 7,  # time to live in seconds (1 week)
-                b85encode(pickle.dumps(session, PICKLE_PROTOCOL)),
-            )
-        )
-
     def write_error(self, status_code: int, **kwargs: Any) -> None:
         """Respond with error message."""
         if "exc_info" in kwargs:
@@ -273,8 +282,9 @@ class Backdoor(APIRequestHandler):
             if not issubclass(exc_info[0], HTTPError):
                 self.finish(
                     pickle.dumps(
-                        self.get_error_message(**kwargs), PICKLE_PROTOCOL
+                        self.get_error_message(**kwargs),
+                        self.get_protocol_version(),
                     )
                 )
                 return
-        self.finish(pickle.dumps(None, PICKLE_PROTOCOL))
+        self.finish(pickle.dumps(None, self.get_protocol_version()))
