@@ -17,29 +17,31 @@
 from __future__ import annotations
 
 import asyncio
-import configparser
 import importlib
 import logging
 import os
-import re
 import signal
 import ssl
 import sys
 import types
-from asyncio.runners import _cancel_all_tasks  # type: ignore
+from asyncio import AbstractEventLoop
+from asyncio.runners import _cancel_all_tasks  # type: ignore[attr-defined]
 from base64 import b64encode
 from collections.abc import Callable, Coroutine
+from configparser import ConfigParser
 from hashlib import sha256
 from multiprocessing import process
 from pathlib import Path
 from typing import Any, cast
+from zoneinfo import ZoneInfo
 
 import orjson
+import regex
 import tornado.netutil
 import tornado.process
 from ecs_logging import StdlibFormatter
-from elastic_enterprise_search import AppSearch  # type: ignore
-from elasticapm.contrib.tornado import ElasticAPM  # type: ignore
+from elastic_enterprise_search import AppSearch  # type: ignore[import]
+from elasticapm.contrib.tornado import ElasticAPM  # type: ignore[import]
 from elasticsearch import AsyncElasticsearch, NotFoundError
 from redis.asyncio import (
     BlockingConnectionPool,
@@ -65,12 +67,13 @@ from .contact.contact import apply_contact_stuff_to_app
 from .quotes import AUTHORS_CACHE, QUOTES_CACHE, WRONG_QUOTES_CACHE
 from .utils import static_file_handling
 from .utils.base_request_handler import BaseRequestHandler
+from .utils.logging import WebhookFormatter, WebhookHandler
 from .utils.request_handler import NotFoundHandler
 from .utils.static_file_handling import StaticFileHandler
 from .utils.utils import Handler, ModuleInfo, Permission, Timer, time_function
 
 IGNORED_MODULES = {
-    "backdoor.backdoor_client",
+    "backdoor.client",
     "patches.*",
     "quotes.quotes_image",
     "quotes.share_page",
@@ -78,12 +81,12 @@ IGNORED_MODULES = {
     "swapped_words.sw_config_file",
     "templates.*",
     "utils.base_request_handler",
-    "utils.braille",
+    "utils.logging",
     "utils.static_file_handling",
     "utils.utils",
 }
 
-CONFIG = configparser.ConfigParser(interpolation=None)
+CONFIG = ConfigParser(interpolation=None)
 CONFIG.read("config.ini", encoding="utf-8")
 
 logger = logging.getLogger(__name__)
@@ -190,9 +193,7 @@ def sort_module_infos(module_infos: list[ModuleInfo]) -> None:
             break
 
 
-def get_all_handlers(
-    module_infos: tuple[ModuleInfo, ...],
-) -> list[Handler]:
+def get_all_handlers(module_infos: tuple[ModuleInfo, ...]) -> list[Handler]:
     """
     Parse the module information and return the handlers in a tuple.
 
@@ -238,6 +239,7 @@ def get_all_handlers(
 
     # redirect handler, to make finding APIs easier
     handlers.append((r"(?i)/(.+)/api/*", RedirectHandler, {"url": "/api/{0}"}))
+
     # redirect from /api to /api/endpunkte (not with alias, because it fails)
     handlers.append((r"(?i)/api/*", RedirectHandler, {"url": "/api/endpunkte"}))
 
@@ -245,12 +247,22 @@ def get_all_handlers(
         (r"/.well-known/(.*)", StaticFileHandler, {"path": ".well-known"})
     )
 
-    logger.debug("Loaded %d handlers.", len(handlers))
+    logger.debug("Loaded %d handlers", len(handlers))
 
     return handlers
 
 
-def make_app() -> str | Application:
+def ignore_modules(config: ConfigParser) -> None:
+    """Read ignored modules from the config."""
+    for module_name in config.get(
+        "GENERAL", "IGNORED_MODULES", fallback=""
+    ).split(","):
+        module_name = module_name.strip()  # pylint: disable=redefined-loop-name
+        if len(module_name) > 0:
+            IGNORED_MODULES.add(module_name)
+
+
+def make_app(config: ConfigParser) -> str | Application:
     """Create the Tornado application and return it."""
     module_infos, duration = time_function(get_module_infos)
     if isinstance(module_infos, str):
@@ -269,22 +281,19 @@ def make_app() -> str | Application:
         autoreload=False,
         debug=bool(sys.flags.dev_mode),
         default_handler_class=NotFoundHandler,
+        compress_response=config.getboolean(
+            "GENERAL", "COMPRESS_RESPONSE", fallback=False
+        ),
         websocket_ping_interval=10,
         # Template settings
         template_path=TEMPLATES_DIR,
     )
 
 
-def apply_config_to_app(
-    app: Application, config: configparser.ConfigParser
-) -> None:
+def apply_config_to_app(app: Application, config: ConfigParser) -> None:
     """Apply the config (from the config.ini file) to the application."""
     app.settings["CONFIG"] = config
 
-    app.settings["compress_response"] = config.getboolean(
-        "GENERAL", "COMPRESS_RESPONSE", fallback=False
-    )
-
     app.settings["cookie_secret"] = config.get(
         "GENERAL", "COOKIE_SECRET", fallback=b"xyzzy"
     )
@@ -352,8 +361,8 @@ def apply_config_to_app(
     apply_contact_stuff_to_app(app, config)
 
 
-def get_ssl_context(
-    config: configparser.ConfigParser,
+def get_ssl_context(  # pragma: no cover
+    config: ConfigParser,
 ) -> None | ssl.SSLContext:
     """Create SSL context and configure using the config."""
     if config.getboolean("TLS", "ENABLED", fallback=False):
@@ -367,14 +376,18 @@ def get_ssl_context(
     return None
 
 
-def setup_logging(
-    config: configparser.ConfigParser, *, force: bool = False
+def setup_logging(  # pragma: no cover
+    config: ConfigParser,
+    loop: None | AbstractEventLoop = None,
+    *,
+    force: bool = False,
 ) -> None:
-    """Configure logging."""
-    debug = config.getboolean("LOGGING", "DEBUG", fallback=sys.flags.dev_mode)
-    path = config.get("LOGGING", "PATH", fallback=None)
-
+    """Setup logging."""  # noqa: D401
     root_logger = logging.getLogger()
+    debug = config.getboolean("LOGGING", "DEBUG", fallback=sys.flags.dev_mode)
+    default_logging_format_no_color = regex.sub(
+        r"%\((end_)?color\)s", "", LogFormatter.DEFAULT_FORMAT
+    )
 
     if root_logger.handlers:
         if not force:
@@ -388,7 +401,7 @@ def setup_logging(
     stream_handler = logging.StreamHandler()
     stream_handler.setFormatter(
         logging.Formatter(
-            re.sub(r"%\((end_)?color\)s", "", LogFormatter.DEFAULT_FORMAT),
+            default_logging_format_no_color,
             LogFormatter.DEFAULT_DATE_FORMAT,
         )
         if sys.flags.dev_mode
@@ -396,6 +409,7 @@ def setup_logging(
     )
     root_logger.addHandler(stream_handler)
 
+    path = config.get("LOGGING", "PATH", fallback=None)
     if path:
         os.makedirs(path, 0o755, True)
         file_handler = logging.handlers.TimedRotatingFileHandler(
@@ -407,13 +421,51 @@ def setup_logging(
         file_handler.setFormatter(StdlibFormatter())
         root_logger.addHandler(file_handler)
 
+    webhook_url = config.get("LOGGING", "WEBHOOK_URL", fallback=None)
+    if webhook_url and loop:
+        webhook_content_type = config.get(
+            "LOGGING",
+            "WEBHOOK_CONTENT_TYPE",
+            fallback="application/json",
+        )
+        webhook_handler = WebhookHandler(
+            logging.ERROR,
+            loop=loop,
+            url=webhook_url,
+            content_type=webhook_content_type,
+        )
+        formatter = WebhookFormatter(
+            config.get(
+                "LOGGING",
+                "WEBHOOK_BODY_FORMAT",
+                fallback='{"text":"' + default_logging_format_no_color + '"}',
+            ),
+            config.get(
+                "LOGGING",
+                "WEBHOOK_TIMESTAMP_FORMAT",
+                fallback=None,
+            ),
+        )
+        formatter.timezone = (
+            ZoneInfo(config.get("LOGGING", "WEBHOOK_TIMESTAMP_TIMEZONE"))
+            if config.has_option("LOGGING", "WEBHOOK_TIMESTAMP_TIMEZONE")
+            else None
+        )
+        formatter.escape_message = config.getboolean(
+            "LOGGING",
+            "WEBHOOK_ESCAPE_MESSAGE",
+            fallback=True,
+        )
+        webhook_handler.setFormatter(formatter)
+        root_logger.addHandler(webhook_handler)
+
     logging.captureWarnings(True)
 
     logging.getLogger("tornado.curl_httpclient").setLevel(logging.INFO)
     logging.getLogger("elasticsearch").setLevel(logging.INFO)
 
 
-def setup_apm(app: Application) -> None:
+def setup_apm(app: Application) -> None:  # pragma: no cover
     """Setup APM."""  # noqa: D401
     config = app.settings["CONFIG"]
     app.settings["ELASTIC_APM"] = {
@@ -439,6 +491,7 @@ def setup_apm(app: Application) -> None:
         "TRANSACTION_IGNORE_URLS": [
             "/api/ping",
             "/static/*",
+            "/favicon.png",
         ],
         "TRANSACTIONS_IGNORE_PATTERNS": ["^OPTIONS "],
         "PROCESSORS": [
@@ -463,26 +516,31 @@ def setup_apm(app: Application) -> None:
             app.settings["ELASTIC_APM"]["INLINE_SCRIPT"].encode("ascii")
         ).digest()
     ).decode("ascii")
-    app.settings["ELASTIC_APM_CLIENT"] = ElasticAPM(app).client
+    app.settings["ELASTIC_APM"]["CLIENT"] = ElasticAPM(app).client
 
 
-def setup_app_search(app: Application) -> None:
+def setup_app_search(app: Application) -> None:  # pragma: no cover
     """Setup Elastic App Search."""  # noqa: D401
     config = app.settings["CONFIG"]
-    app.settings["APP_SEARCH"] = AppSearch(
-        config.get("APP_SEARCH", "HOST", fallback=None),
-        http_auth=config.get("APP_SEARCH", "SEARCH_KEY", fallback=None),
-        verify_cert=config.getboolean(
-            "APP_SEARCH", "VERIFY_CERT", fallback=True
-        ),
-        ca_certs=os.path.join(DIR, "ca-bundle.crt"),
+    host = config.get("APP_SEARCH", "HOST", fallback=None)
+    key = config.get("APP_SEARCH", "SEARCH_KEY", fallback=None)
+    app.settings["APP_SEARCH"] = (
+        AppSearch(
+            host,
+            http_auth=key,
+            verify_cert=config.getboolean(
+                "APP_SEARCH", "VERIFY_CERT", fallback=True
+            ),
+            ca_certs=os.path.join(DIR, "ca-bundle.crt"),
+        )
+        if host
+        else None
     )
-    app.settings["APP_SEARCH_ENGINE_NAME"] = config.get(
+    app.settings["APP_SEARCH_HOST"] = host
+    app.settings["APP_SEARCH_KEY"] = key
+    app.settings["APP_SEARCH_ENGINE"] = config.get(
         "APP_SEARCH", "ENGINE_NAME", fallback=NAME.removesuffix("-dev")
     )
-    app.settings["APP_SEARCH_SEARCH_KEY"] = config.get(
-        "APP_SEARCH", "SEARCH_KEY", fallback=None
-    )
 
 
 def setup_elasticsearch(app: Application) -> None | AsyncElasticsearch:
@@ -526,7 +584,7 @@ def setup_elasticsearch(app: Application) -> None | AsyncElasticsearch:
     return elasticsearch
 
 
-async def check_elasticsearch(app: Application) -> None:
+async def check_elasticsearch(app: Application) -> None:  # pragma: no cover
     """Check Elasticsearch."""
     # pylint: disable=invalid-name
     while True:  # pylint: disable=while-used
@@ -535,10 +593,9 @@ async def check_elasticsearch(app: Application) -> None:
         )
         try:
             await es.transport.perform_request("HEAD", "/")
-        except Exception as exc:  # pylint: disable=broad-except
+        except Exception:  # pylint: disable=broad-except
             EVENT_ELASTICSEARCH.clear()
-            logger.exception(exc)
-            logger.error("Connecting to Elasticsearch failed!")
+            logger.exception("Connecting to Elasticsearch failed")
         else:
             if not EVENT_ELASTICSEARCH.is_set():
                 await setup_elasticsearch_configs(
@@ -557,7 +614,6 @@ async def setup_elasticsearch_configs(  # noqa: C901
     get: Callable[..., Coroutine[Any, Any, dict[str, Any]]]
     put: Callable[..., Coroutine[Any, Any, dict[str, Any]]]
     for i in range(3):
-
         if i == 0:  # pylint: disable=compare-to-zero
             what = "ingest_pipelines"
             get = elasticsearch.ingest.get_pipeline
@@ -572,11 +628,9 @@ async def setup_elasticsearch_configs(  # noqa: C901
             put = elasticsearch.indices.put_index_template
 
         base_path = Path(os.path.join(DIR, "elasticsearch", what))
-
         for path in base_path.rglob("*.json"):
-
             if not path.is_file():
-                logger.warning("%s is not a file!", path)
+                logger.warning("%s is not a file", path)
                 continue
 
             body = orjson.loads(
@@ -666,22 +720,21 @@ def setup_redis(app: Application) -> None | Redis[str]:
     return redis
 
 
-async def check_redis(app: Application) -> None:
+async def check_redis(app: Application) -> None:  # pragma: no cover
     """Check Redis."""
     while True:  # pylint: disable=while-used
         redis: Redis[str] = cast("Redis[str]", app.settings.get("REDIS"))
         try:
             await redis.ping()
-        except Exception as exc:  # pylint: disable=broad-except
+        except Exception:  # pylint: disable=broad-except
             EVENT_REDIS.clear()
-            logger.exception(exc)
-            logger.error("Connecting to Redis failed!")
+            logger.exception("Connecting to Redis failed")
         else:
             EVENT_REDIS.set()
         await asyncio.sleep(20)
 
 
-async def wait_for_shutdown() -> None:
+async def wait_for_shutdown() -> None:  # pragma: no cover
     """Wait for the shutdown event."""
     loop = asyncio.get_running_loop()
     while not EVENT_SHUTDOWN.is_set():  # pylint: disable=while-used
@@ -689,7 +742,7 @@ async def wait_for_shutdown() -> None:
     loop.stop()
 
 
-def signal_handler(  # noqa: D103
+def signal_handler(  # noqa: D103  # pragma: no cover
     signalnum: int, frame: None | types.FrameType
 ) -> None:
     # pylint: disable=unused-argument, missing-function-docstring
@@ -699,13 +752,13 @@ def signal_handler(  # noqa: D103
         EVENT_SHUTDOWN.set()
 
 
-def install_signal_handler() -> None:
+def install_signal_handler() -> None:  # pragma: no cover
     """Install the signal handler."""
     signal.signal(signal.SIGINT, signal_handler)
     signal.signal(signal.SIGTERM, signal_handler)
 
 
-def main() -> None | int | str:  # noqa: C901
+def main() -> None | int | str:  # noqa: C901  # pragma: no cover
     """
     Start everything.
 
@@ -716,24 +769,20 @@ def main() -> None | int | str:  # noqa: C901
 
     install_signal_handler()
 
-    setup_logging(CONFIG)
+    loop = asyncio.get_event_loop_policy().get_event_loop()
+
+    setup_logging(CONFIG, loop)
 
     logger.info("Starting %s %s", NAME, VERSION)
 
     if sys.platform == "win32":
         logger.warning(
-            "Please note that running on Windows is not officially supported."
+            "Please note that running on Windows is not officially supported"
         )
 
-    # read ignored modules from the config
-    for module_name in CONFIG.get(
-        "GENERAL", "IGNORED_MODULES", fallback=""
-    ).split(","):
-        module_name = module_name.strip()  # pylint: disable=redefined-loop-name
-        if len(module_name) > 0:
-            IGNORED_MODULES.add(module_name)
+    ignore_modules(CONFIG)
 
-    app = make_app()
+    app = make_app(CONFIG)
     if isinstance(app, str):
         return app
 
@@ -741,7 +790,9 @@ def main() -> None | int | str:  # noqa: C901
     setup_elasticsearch(app)
     setup_app_search(app)
     setup_redis(app)
-    setup_apm(app)
+
+    if CONFIG.getboolean("ELASTIC_APM", "ENABLED", fallback=False):
+        setup_apm(app)
 
     behind_proxy = CONFIG.getboolean("GENERAL", "BEHIND_PROXY", fallback=False)
 
@@ -804,8 +855,6 @@ def main() -> None | int | str:  # noqa: C901
             )
         )
 
-    loop = asyncio.get_event_loop_policy().get_event_loop()
-
     _ = asyncio.get_event_loop
     asyncio.get_event_loop = lambda: loop
     server.add_sockets(sockets)
