@@ -18,32 +18,33 @@ from __future__ import annotations
 
 import asyncio
 import configparser
+import contextlib
 import http.client
 import json as stdlib_json  # pylint: disable=preferred-module
 import os
 import sys
+from collections.abc import Callable
+from pathlib import Path
 from typing import Any
 
 import certifi
-import defusedxml  # type: ignore
-import namedthreads  # type: ignore
+import defusedxml  # type: ignore[import]
+import namedthreads  # type: ignore[import]
 import tornado.httpclient
 import tornado.httputil
 import tornado.log
 import tornado.web
 import uvloop
 import yaml
-from tornado.httputil import parse_body_arguments as _parse_body_arguments
 
 from .. import DIR as ROOT_DIR
 from . import braille, json  # noqa: F401  # pylint: disable=reimported
 
 DIR = os.path.dirname(__file__)
 
-try:
-    from jxlpy import JXLImagePlugin  # type: ignore  # noqa: F401
-except ImportError:
-    pass
+with contextlib.suppress(ImportError):
+    # pylint: disable=import-error, useless-suppression
+    from jxlpy import JXLImagePlugin  # type: ignore[import]  # noqa: F401
 
 
 def apply() -> None:
@@ -53,8 +54,16 @@ def apply() -> None:
     defusedxml.defuse_stdlib()
     defusedxml.xmlrpc.monkey_patch()
     certifi.where = lambda: os.path.join(ROOT_DIR, "ca-bundle.crt")
+    certifi.contents = lambda: Path(certifi.where()).read_text("ascii")
     configparser.RawConfigParser.BOOLEAN_STATES.update(  # type: ignore[attr-defined]
-        {"sure": True, "nope": False}
+        {
+            "sure": True,
+            "nope": False,
+            "accept": True,
+            "reject": False,
+            "enabled": True,
+            "disabled": False,
+        }
     )
     asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
     tornado.httpclient.AsyncHTTPClient.configure(
@@ -75,6 +84,13 @@ def apply() -> None:
     tornado.web.RequestHandler.propfind = _  # type: ignore[attr-defined]
     tornado.web.RequestHandler.brew = _  # type: ignore[attr-defined]
     tornado.web.RequestHandler.when = _  # type: ignore[attr-defined]
+    tornado.web.GZipContentEncoding.CONTENT_TYPES.add("application/x-ndjson")
+    tornado.web.GZipContentEncoding.CONTENT_TYPES.add("application/yaml")
+    tornado.web.GZipContentEncoding._compressible_type = (  # type: ignore[assignment]
+        lambda self, ctype: ctype in self.CONTENT_TYPES
+        or ctype.endswith(("+xml", "+json"))
+        or ctype.startswith("text/")
+    )
     http.client.responses[420] = "Enhance Your Calm"
     if not getattr(stdlib_json, "_omegajson", False):
         patch_json()
@@ -129,14 +145,16 @@ def ensure_bytes(value: Any) -> bytes:
     return str(value).encode("utf-8")
 
 
-def parse_body_arguments(  # noqa: D103,C901,B950 # pylint: disable=too-complex,too-many-branches
+def parse_body_arguments(  # noqa: D103, C901
     content_type: str,
     body: bytes,
     arguments: dict[str, list[bytes]],
     files: dict[str, list[tornado.httputil.HTTPFile]],
     headers: None | tornado.httputil.HTTPHeaders = None,
+    *,
+    _: Callable[..., None] = tornado.httputil.parse_body_arguments,
 ) -> None:
-    # pylint: disable=missing-function-docstring
+    # pylint: disable=missing-function-docstring, too-complex, too-many-branches
     if content_type.startswith("application/json"):
         if headers and "Content-Encoding" in headers:
             tornado.log.gen_log.warning(
@@ -166,7 +184,7 @@ def parse_body_arguments(  # noqa: D103,C901,B950 # pylint: disable=too-complex,
                 if value is not None:
                     arguments.setdefault(key, []).append(ensure_bytes(value))
     else:
-        _parse_body_arguments(content_type, body, arguments, files, headers)
+        _(content_type, body, arguments, files, headers)
 
 
 def redirect(
