@@ -19,8 +19,14 @@ import asyncio
 import io
 import os
 import sys
+from asyncio import Future
 from queue import SimpleQueue
-from tempfile import NamedTemporaryFile, TemporaryDirectory
+from tempfile import (  # pylint: disable=import-private-name
+    NamedTemporaryFile,
+    TemporaryDirectory,
+    _TemporaryFileWrapper,
+)
+from typing import Any
 from urllib.parse import unquote
 
 from tornado.web import stream_request_body
@@ -64,10 +70,18 @@ class UpdateAPI(APIRequestHandler):
     ALLOWED_METHODS: tuple[str, ...] = ("PUT",)
     REQUIRED_PERMISSION = Permission.UPDATE
 
+    dir: TemporaryDirectory[str]
+    file: _TemporaryFileWrapper[bytes]
     queue: SimpleQueue[None | bytes]
+    future: Future[Any]
 
-    async def prepare(self) -> None:
-        # pylint: disable=attribute-defined-outside-init, consider-using-with
+    def data_received(self, chunk: bytes) -> None:  # noqa: D102
+        self.queue.put(chunk)
+
+    def on_finish(self) -> None:  # noqa: D102
+        self.queue.put(None)
+
+    async def prepare(self) -> None:  # noqa: D102
         await super().prepare()
         loop = asyncio.get_running_loop()
         self.dir = TemporaryDirectory()
@@ -77,12 +91,6 @@ class UpdateAPI(APIRequestHandler):
             None, write_from_queue, self.file, self.queue
         )
 
-    def data_received(self, chunk: bytes) -> None:
-        self.queue.put(chunk)
-
-    def on_finish(self) -> None:
-        self.queue.put(None)
-
     async def put(self, filename: str) -> None:
         """Handle PUT requests to the update API."""
         # pylint: disable=while-used
