@@ -1,10 +1,12 @@
 // @license magnet:?xt=urn:btih:0b31508aeb0634b347b8270c7bee4d411b5d4109&dn=agpl-3.0.txt GNU-AGPL-3.0-or-later
-function startDisplayingUptime(uptimeAtStart) {
+function startDisplayingUptime() {
     const uptimeDiv = elById("uptime");
     // set the font family, because it looks much better
     uptimeDiv.style.fontFamily = "'clock-face', monospace";
     // the time the website got loaded minus the uptime
-    const startTime = (performance.now() / 1000) - uptimeAtStart;
+    const startTime = (performance.now() / 1000) - parseFloat(
+        uptimeDiv.getAttribute("uptime")
+    );
 
     const zeroPad = n => String(Math.floor(n)).padStart(2, "0");
 
@@ -21,9 +23,11 @@ function startDisplayingUptime(uptimeAtStart) {
             zeroPad(div_60 % 60),  // minutes
             zeroPad(uptime % 60)  // seconds
         ].join(":");
+        uptimeDiv.setAttribute("uptime", String(uptime));
     };
 
     displayUptime();
     setInterval(displayUptime, 1000);
 }
+startDisplayingUptime();
 // @license-end
