@@ -21,9 +21,7 @@ from __future__ import annotations
 
 import logging
 import os
-import re
 import sys
-import time
 from asyncio import Future
 from datetime import date, datetime, timedelta
 from http.client import responses
@@ -31,6 +29,7 @@ from typing import Any
 from urllib.parse import unquote, urlsplit
 
 import html2text
+import regex
 from ansi2html import Ansi2HTMLConverter
 from bs4 import BeautifulSoup
 from dateutil.easter import easter
@@ -43,6 +42,7 @@ from .. import GH_ORG_URL, GH_PAGES_URL, GH_REPO_URL
 from .base_request_handler import BaseRequestHandler
 from .static_file_handling import fix_static_url
 from .utils import (
+    SUS_PATHS,
     ModuleInfo,
     Permission,
     bool_to_str,
@@ -90,116 +90,6 @@ class HTMLRequestHandler(BaseRequestHandler):
     )
     used_render = False
 
-    def get_form_appendix(self) -> str:
-        """Get HTML to add to forms to keep important query args."""
-        form_appendix = (
-            "<input name='no_3rd_party' class='hidden' "
-            f"value='{bool_to_str(self.get_no_3rd_party())}'>"
-            if "no_3rd_party" in self.request.query_arguments
-            and self.get_no_3rd_party() != self.get_saved_no_3rd_party()
-            else ""
-        )
-        if self.get_dynload() != self.get_saved_dynload():
-            form_appendix += (
-                "<input name='dynload' class='hidden' "
-                f"value='{bool_to_str(self.get_dynload())}'>"
-            )
-        if (theme := self.get_theme()) != self.get_saved_theme():
-            form_appendix += (
-                f"<input name='theme' class='hidden' value='{theme}'>"
-            )
-        return form_appendix
-
-    def write_error(self, status_code: int, **kwargs: Any) -> None:
-        """Render the error page with the status_code as a HTML page."""
-        self.render(
-            "error.html",
-            status=status_code,
-            reason=self.get_error_message(**kwargs),
-            description=self.get_error_page_description(status_code),
-            is_traceback="exc_info" in kwargs
-            and not issubclass(kwargs["exc_info"][0], HTTPError)
-            and (
-                self.settings.get("serve_traceback")
-                or self.is_authorized(Permission.TRACEBACK)
-            ),
-        )
-
-    def get_template_namespace(self) -> dict[str, Any]:
-        """
-        Add useful things to the template namespace and return it.
-
-        They are mostly needed by most of the pages (like title,
-        description and no_3rd_party).
-        """
-        namespace = super().get_template_namespace()
-        namespace.update(
-            ansi2html=Ansi2HTMLConverter(inline=True, scheme="xterm"),
-            as_html=self.content_type == "text/html",
-            c=self.settings.get("TESTING")
-            or self.now.date() == date(self.now.year, 4, 1)
-            or str_to_bool(self.get_cookie("c", "f") or "f", False),
-            canonical_url=self.fix_url(
-                self.request.full_url().upper()
-                if self.request.path.upper().startswith("/LOLWUT")
-                else self.request.full_url().lower()
-            ).split("?")[0],
-            description=self.description,
-            dynload=self.get_dynload(),
-            elastic_rum_url=self.ELASTIC_RUM_URL,
-            fix_static=lambda url: self.fix_url(fix_static_url(url)),
-            fix_url=self.fix_url,
-            openmoji=self.get_openmoji(),
-            emoji2html=(
-                lambda emoji: create_emoji_html(
-                    emoji,
-                    self.fix_url(
-                        fix_static_url(
-                            f"img/openmoji-svg/{emoji2code(emoji)}.svg"
-                        )
-                    ),
-                )
-            )
-            if self.get_openmoji()
-            else lambda spam: spam,  # type: ignore[no-any-return]
-            form_appendix=self.get_form_appendix(),
-            GH_ORG_URL=GH_ORG_URL,
-            GH_PAGES_URL=GH_PAGES_URL,
-            GH_REPO_URL=GH_REPO_URL,
-            keywords="Asoziales Netzwerk, Känguru-Chroniken"
-            + (
-                f", {self.module_info.get_keywords_as_str(self.request.path)}"
-                if self.module_info
-                else ""
-            ),
-            lang="de",  # TODO: add language support
-            no_3rd_party=self.get_no_3rd_party(),
-            now=self.now,
-            settings=self.settings,
-            short_title=self.short_title,
-            theme=self.get_display_theme(),
-            title=self.title,
-            apm_script=self.settings["ELASTIC_APM"]["INLINE_SCRIPT"]
-            if self.elastic_apm_enabled()
-            else None,
-        )
-        namespace.update(
-            {
-                "🥚": self.settings.get("TESTING")
-                or timedelta()
-                <= self.now.date() - easter(self.now.year)
-                < timedelta(days=2),
-                "🦘": self.settings.get("TESTING")
-                or isprime(self.now.microsecond),  # type: ignore[no-untyped-call]
-            }
-        )
-        return namespace
-
-    def render(self, template_name: str, **kwargs: Any) -> Future[None]:
-        """Render a template."""
-        self.used_render = True
-        return super().render(template_name, **kwargs)
-
     def finish(
         self, chunk: None | str | bytes | dict[Any, Any] = None
     ) -> Future[None]:
@@ -259,6 +149,116 @@ class HTMLRequestHandler(BaseRequestHandler):
         )
         return super().finish(dictionary)
 
+    def get_form_appendix(self) -> str:
+        """Get HTML to add to forms to keep important query args."""
+        form_appendix = (
+            "<input name='no_3rd_party' class='hidden' "
+            f"value='{bool_to_str(self.get_no_3rd_party())}'>"
+            if "no_3rd_party" in self.request.query_arguments
+            and self.get_no_3rd_party() != self.get_saved_no_3rd_party()
+            else ""
+        )
+        if self.get_dynload() != self.get_saved_dynload():
+            form_appendix += (
+                "<input name='dynload' class='hidden' "
+                f"value='{bool_to_str(self.get_dynload())}'>"
+            )
+        if (theme := self.get_theme()) != self.get_saved_theme():
+            form_appendix += (
+                f"<input name='theme' class='hidden' value='{theme}'>"
+            )
+        return form_appendix
+
+    def get_template_namespace(self) -> dict[str, Any]:
+        """
+        Add useful things to the template namespace and return it.
+
+        They are mostly needed by most of the pages (like title,
+        description and no_3rd_party).
+        """
+        namespace = super().get_template_namespace()
+        namespace.update(
+            ansi2html=Ansi2HTMLConverter(inline=True, scheme="xterm"),
+            as_html=self.content_type == "text/html",
+            c=self.settings.get("TESTING")
+            or self.now.date() == date(self.now.year, 4, 1)
+            or str_to_bool(self.get_cookie("c", "f") or "f", False),
+            canonical_url=self.fix_url(
+                self.request.full_url().upper()
+                if self.request.path.upper().startswith("/LOLWUT")
+                else self.request.full_url().lower()
+            ).split("?")[0],
+            description=self.description,
+            dynload=self.get_dynload(),
+            elastic_rum_url=self.ELASTIC_RUM_URL,
+            fix_static=lambda url: self.fix_url(fix_static_url(url)),
+            fix_url=self.fix_url,
+            openmoji=self.get_openmoji(),
+            emoji2html=(
+                lambda emoji: create_emoji_html(
+                    emoji,
+                    self.fix_url(
+                        fix_static_url(
+                            f"img/openmoji-svg-14.0/{emoji2code(emoji)}.svg"
+                        )
+                    ),
+                )
+            )
+            if self.get_openmoji() == "img"
+            else lambda spam: spam,  # type: ignore[no-any-return]
+            form_appendix=self.get_form_appendix(),
+            GH_ORG_URL=GH_ORG_URL,
+            GH_PAGES_URL=GH_PAGES_URL,
+            GH_REPO_URL=GH_REPO_URL,
+            keywords="Asoziales Netzwerk, Känguru-Chroniken"
+            + (
+                f", {self.module_info.get_keywords_as_str(self.request.path)}"
+                if self.module_info
+                else ""
+            ),
+            lang="de",  # TODO: add language support
+            no_3rd_party=self.get_no_3rd_party(),
+            now=self.now,
+            settings=self.settings,
+            short_title=self.short_title,
+            theme=self.get_display_theme(),
+            title=self.title,
+            apm_script=self.settings["ELASTIC_APM"].get("INLINE_SCRIPT")
+            if self.apm_enabled
+            else None,
+        )
+        namespace.update(
+            {
+                "🥚": self.settings.get("TESTING")
+                or timedelta()
+                <= self.now.date() - easter(self.now.year)
+                < timedelta(days=2),
+                "🦘": self.settings.get("TESTING")
+                or isprime(self.now.microsecond),  # type: ignore[no-untyped-call]
+            }
+        )
+        return namespace
+
+    def render(self, template_name: str, **kwargs: Any) -> Future[None]:
+        """Render a template."""
+        self.used_render = True
+        return super().render(template_name, **kwargs)
+
+    def write_error(self, status_code: int, **kwargs: Any) -> None:
+        """Render the error page with the status_code as a HTML page."""
+        self.render(
+            "error.html",
+            status=status_code,
+            reason=self.get_error_message(**kwargs),
+            description=self.get_error_page_description(status_code),
+            is_traceback="exc_info" in kwargs
+            and not issubclass(kwargs["exc_info"][0], HTTPError)
+            and (
+                self.settings.get("serve_traceback")
+                or self.is_authorized(Permission.TRACEBACK)
+            ),
+        )
+
 
 class APIRequestHandler(BaseRequestHandler):
     """
@@ -302,7 +302,7 @@ class NotFoundHandler(HTMLRequestHandler):
 
         if self.request.method not in {"GET", "HEAD"}:
             raise HTTPError(404)
-        new_path = re.sub(r"/+", "/", self.request.path.rstrip("/")).replace(
+        new_path = regex.sub(r"/+", "/", self.request.path.rstrip("/")).replace(
             "_", "-"
         )
         for ext in (".html", ".htm", ".php"):
@@ -311,30 +311,7 @@ class NotFoundHandler(HTMLRequestHandler):
 
         new_path = replace_umlauts(new_path)
 
-        if new_path.lower() in {
-            "/-profiler/phpinfo",
-            "/.aws/credentials",
-            "/.env.bak",
-            "/.ftpconfig",
-            "/admin/controller/extension/extension",
-            "/assets/filemanager/dialog",
-            "/assets/vendor/server/php",
-            "/aws.yml",
-            "/boaform/admin/formlogin",
-            "/phpinfo",
-            "/public/assets/jquery-file-upload/server/php",
-            "/root",
-            "/settings/aws.yml",
-            "/uploads",
-            "/vendor/phpunit/phpunit/src/util/php/eval-stdin",
-            "/wordpress",
-            "/wp",
-            "/wp-admin",
-            "/wp-admin/css",
-            "/wp-includes",
-            "/wp-login",
-            "/wp-upload",
-        }:
+        if new_path.lower() in SUS_PATHS:
             self.set_status(469, reason="Nice Try")
             return self.write_error(469)
 
@@ -414,7 +391,7 @@ class ZeroDivision(HTMLRequestHandler):
         """Divide by zero and raise an error."""
         self.now = await self.get_time()
         self.handle_accept_header(self.POSSIBLE_CONTENT_TYPES)
-        if not self.request.method == "OPTIONS":
+        if self.request.method != "OPTIONS":
             420 / 0  # pylint: disable=pointless-statement
 
 
@@ -431,8 +408,7 @@ class ElasticRUM(BaseRequestHandler):
         "/dist/bundles/elastic-apm-rum.umd{}.js{}"
     )
 
-    SCRIPTS: dict[str, tuple[str, float]] = {}
-    CACHE_TIME = 365 * 60 * 60 * 24
+    SCRIPTS: dict[str, str] = {}
 
     async def get(
         self,
@@ -448,7 +424,7 @@ class ElasticRUM(BaseRequestHandler):
         self.handle_accept_header((accepted_ct,))
 
         key = version + spam + eggs
-        if key not in self.SCRIPTS or self.SCRIPTS[key][1] < time.monotonic():
+        if key not in self.SCRIPTS:
             response = await AsyncHTTPClient().fetch(
                 self.URL.format(version, spam, eggs),
                 raise_error=False,
@@ -456,10 +432,7 @@ class ElasticRUM(BaseRequestHandler):
             )
             if response.code != 200:
                 raise HTTPError(response.code, reason=response.reason)
-            self.SCRIPTS[key] = (
-                response.body.decode(),
-                time.monotonic() + 365 * 24 * 60 * 60,
-            )
+            self.SCRIPTS[key] = response.body.decode()
             new_path = urlsplit(response.effective_url).path
             if new_path.endswith(".js"):
                 BaseRequestHandler.ELASTIC_RUM_URL = new_path
@@ -470,10 +443,9 @@ class ElasticRUM(BaseRequestHandler):
             self.set_header(
                 "SourceMap", self.request.full_url().split("?")[0] + ".map"
             )
+        self.set_header("Expires", datetime.utcnow() + timedelta(days=365))
         self.set_header(
-            "Expires", datetime.utcnow() + timedelta(seconds=self.CACHE_TIME)
+            "Cache-Control",
+            f"public, min-fresh={60 * 60 * 24 * 365}, immutable",
         )
-        self.set_header(
-            "Cache-Control", f"public, min-fresh={self.CACHE_TIME}, immutable"
-        )
-        return await self.finish(self.SCRIPTS[key][0])
+        return await self.finish(self.SCRIPTS[key])
