@@ -18,7 +18,6 @@ from __future__ import annotations
 import asyncio
 import os
 import random
-import re
 import time
 from base64 import b85encode
 from collections.abc import Callable, Sequence
@@ -27,13 +26,15 @@ from datetime import datetime
 from enum import IntFlag
 from functools import cache
 from ipaddress import IPv4Address, IPv6Address, ip_address, ip_network
-from typing import IO, Any, TypeVar, Union
+from typing import IO, Any, Literal, TypeVar, Union
 from urllib.parse import SplitResult, parse_qsl, urlencode, urlsplit, urlunsplit
 
-import elasticapm  # type: ignore
-from blake3 import blake3  # type: ignore
+import elasticapm  # type: ignore[import]
+import regex
+from blake3 import blake3  # type: ignore[import]
 from elasticsearch import AsyncElasticsearch
-from Levenshtein import distance  # type: ignore
+from Levenshtein import distance  # type: ignore[import]
+from redis.asyncio import Redis
 from tornado.web import HTTPError, RequestHandler
 
 from .. import STATIC_DIR
@@ -47,8 +48,46 @@ Handler = Union[
     tuple[str, type[RequestHandler], dict[str, Any], str],
 ]
 
+OpenMojiValue = Literal[False, "img"]  # , "font"]
 
-# sortable so the pages can be linked in an order
+SUS_PATHS = {
+    "/-profiler/phpinfo",
+    "/.aws/credentials",
+    "/.env",
+    "/.env.bak",
+    "/.ftpconfig",
+    "/admin/controller/extension/extension",
+    "/assets/filemanager/dialog",
+    "/assets/vendor/server/php",
+    "/aws.yml",
+    "/boaform/admin/formlogin",
+    "/phpinfo",
+    "/public/assets/jquery-file-upload/server/php",
+    "/root",
+    "/settings/aws.yml",
+    "/uploads",
+    "/vendor/phpunit/phpunit/src/util/php/eval-stdin",
+    "/wordpress",
+    "/wp",
+    "/wp-admin",
+    "/wp-admin/css",
+    "/wp-includes",
+    "/wp-login",
+    "/wp-upload",
+}
+
+
+class Permission(IntFlag):
+    """Permissions for accessing restricted stuff."""
+
+    RATELIMITS = 1
+    TRACEBACK = 2
+    BACKDOOR = 4
+    UPDATE = 8
+    REPORTING = 16
+
+
+# sortable, so the pages can be linked in an order
 # frozen, so it's immutable
 @dataclass(order=True, frozen=True)
 class PageInfo:
@@ -77,7 +116,7 @@ class PageInfo:
         words: Sequence[str]
 
         if isinstance(query, str):
-            words = re.split(r"\s+", query)
+            words = regex.split(r"\s+", query)
             query = query.lower()
             if query in self.description.lower() or query in self.name.lower():
                 score = len(query) / 2
@@ -125,6 +164,14 @@ class ModuleInfo(PageInfo):
     sub_pages: tuple[PageInfo, ...] = field(default_factory=tuple)
     aliases: tuple[str, ...] = field(default_factory=tuple)
 
+    def get_keywords_as_str(self, path: str) -> str:
+        """Get the keywords as comma-seperated string."""
+        page_info = self.get_page_info(path)
+        if self != page_info:
+            return ", ".join((*self.keywords, *page_info.keywords))
+
+        return ", ".join(self.keywords)
+
     def get_page_info(self, path: str) -> PageInfo:
         """Get the PageInfo of the specified path."""
         if self.path == path:
@@ -136,15 +183,7 @@ class ModuleInfo(PageInfo):
 
         return self
 
-    def get_keywords_as_str(self, path: str) -> str:
-        """Get the keywords as comma-seperated string."""
-        page_info = self.get_page_info(path)
-        if self != page_info:
-            return ", ".join((*self.keywords, *page_info.keywords))
-
-        return ", ".join(self.keywords)
-
-    def search(self, query: str | Sequence[str]) -> float:
+    def search(self, query: str | Sequence[str]) -> float:  # noqa: D102
         score = super().search(query)
 
         if len(self.sub_pages) > 0:
@@ -164,6 +203,17 @@ class Timer:
         self._execution_time: None | float = None
         self._start_time: float = time.perf_counter()
 
+    @property
+    def execution_time(self) -> float:
+        """
+        Get the execution time in seconds and return it.
+
+        If the timer wasn't stopped yet a ValueError gets raised.
+        """
+        if self._execution_time is None:
+            raise ValueError("Timer wasn't stopped yet.")
+        return self._execution_time
+
     def stop(self) -> float:
         """
         Stop the timer and return the execution time in seconds.
@@ -177,27 +227,6 @@ class Timer:
         del self._start_time
         return self._execution_time
 
-    @property
-    def execution_time(self) -> float:
-        """
-        Get the execution time in seconds and return it.
-
-        If the timer wasn't stopped yet a ValueError gets raised.
-        """
-        if self._execution_time is None:
-            raise ValueError("Timer wasn't stopped yet.")
-        return self._execution_time
-
-
-class Permission(IntFlag):
-    """Permissions for accessing the website."""
-
-    RATELIMITS = 1
-    TRACEBACK = 2
-    BACKDOOR = 4
-    UPDATE = 8
-    REPORTING = 16
-
 
 @cache
 def add_args_to_url(url: str | SplitResult, **kwargs: dict[str, Any]) -> str:
@@ -258,7 +287,8 @@ def apm_anonymization_processor(  # pylint: disable=unused-argument
     if "context" in event and "request" in event["context"]:
         request = event["context"]["request"]
         if "url" in request and "pathname" in request["url"]:
-            if request["url"]["pathname"] == "/robots.txt":
+            path = request["url"]["pathname"]
+            if path == "/robots.txt" or path.lower() in SUS_PATHS:
                 return event
         if "socket" in request and "remote_address" in request["socket"]:
             request["socket"]["remote_address"] = anonymize_ip(
@@ -268,8 +298,8 @@ def apm_anonymization_processor(  # pylint: disable=unused-argument
             headers = request["headers"]
             if "X-Forwarded-For" in headers:
                 headers["X-Forwarded-For"] = ", ".join(
-                    anonymize_ip(_, ignore_invalid=True)
-                    for _ in headers["X-Forwarded-For"].split(",")
+                    anonymize_ip(ip.strip(), ignore_invalid=True)
+                    for ip in headers["X-Forwarded-For"].split(",")
                 )
             for header in headers:
                 if "ip" in header.lower().split("-"):
@@ -289,14 +319,26 @@ def country_code_to_flag(code: str) -> str:
     return "".join(chr(ord(char) + 23 * 29 * 191) for char in code.upper())
 
 
+def create_emoji_html(emoji: str, emoji_url: str) -> str:
+    """Create an HTML element that can be used to display an emoji."""
+    return f'<img src="{emoji_url}" alt="{emoji}" class="emoji">'
+
+
+def emoji2code(emoji: str) -> str:
+    """Convert an emoji to the hexcodes of it."""
+    return "-".join(f"{ord(c):04x}" for c in emoji).upper()
+
+
 def emojify(string: str) -> str:
     """Emojify a given string."""
-    string = re.sub(
+    string = regex.sub(
         r"[a-zA-Z]+",
         lambda match: "\u200C".join(country_code_to_flag(match[0])),
         replace_umlauts(string),
     )
-    string = re.sub(r"[0-9#*]+", lambda match: f"{'⃣'.join(match[0])}⃣", string)
+    string = regex.sub(
+        r"[0-9#*]+", lambda match: f"{'⃣'.join(match[0])}⃣", string
+    )
     return (
         string.replace("!?", "⁉")
         .replace("!!", "‼")
@@ -317,14 +359,11 @@ async def geoip(
     """Get GeoIP information."""
     if ip not in geoip_cache:
         geoip_cache[ip] = {}
-
     if database not in geoip_cache[ip]:
-
         if not elasticsearch:
             return None
 
         properties: None | tuple[str, ...]
-
         if database == "GeoLite2-City.mmdb":
             properties = (
                 "continent_name",
@@ -383,9 +422,12 @@ def get_themes() -> tuple[str, ...]:
     )
 
 
-def hash_bytes(text: bytes, size: int = 32) -> str:
+def hash_bytes(*args: bytes, size: int = 32) -> str:
     """Hash bytes with BLAKE3 and return the Base85 representation."""
-    digest: bytes = blake3(text).digest(size)
+    hasher = blake3()
+    for spam in args:
+        hasher.update(spam)
+    digest: bytes = hasher.digest(size)
     return b85encode(digest).decode("ascii")
 
 
@@ -394,51 +436,111 @@ def hash_ip(address: str | IPv4Address | IPv6Address) -> str:
     if not isinstance(address, IPv4Address | IPv6Address):
         address = ip_address(address)
     return hash_bytes(
-        blake3(datetime.utcnow().date().isoformat().encode("ascii")).digest()
-        + address.packed
+        blake3(datetime.utcnow().date().isoformat().encode("ascii")).digest(),
+        address.packed,
     )
 
 
-def length_of_match(match: re.Match[str]) -> int:
+def length_of_match(match: regex.Match[str]) -> int:
     """Calculate the length of the regex match and return it."""
     span = match.span()
     return span[1] - span[0]
 
 
-def n_from_set(_set: set[T] | frozenset[T], _n: int) -> set[T]:
-    """Get and return _n elements of the set as a new set."""
-    i = 0
+def n_from_set(set_: set[T] | frozenset[T], n: int) -> set[T]:
+    """Get and return n elements of the set as a new set."""
+    # pylint: disable=invalid-name
     new_set = set()
-    for _el in _set:
-        if i >= _n:
+    for i, element in enumerate(set_):
+        if i >= n:
             break
-        i += 1
-        new_set.add(_el)
+        new_set.add(element)
     return new_set
 
 
 def name_to_id(val: str) -> str:
     """Replace umlauts and whitespaces in a string to get a valid HTML id."""
-    return re.sub(
+    return regex.sub(
         r"[^a-z0-9]+",
         "-",
         replace_umlauts(val).lower(),
     ).strip("-")
 
 
-def remove_suffix_ignore_case(string: str, suffix: str, /) -> str:
-    """Remove a suffix without caring about the case."""
-    if string.lower().endswith(suffix.lower()):
-        return string[: -len(suffix)]
-
-    return string
-
-
 def normalized_levenshtein(string1: str, string2: str) -> float:
     """Calculate the normalized Levenshtein distance between two strings."""
     return float(distance(string1, string2)) / max(len(string1), len(string2))
 
 
+def parse_openmoji_arg(value: str, default: OpenMojiValue) -> OpenMojiValue:
+    """Parse the openmoji arg into a Literal."""
+    value = value.lower()
+    # if value in {"f", "font"}:
+    #     return "font"
+    if value in {"i", "img"}:
+        return "img"
+    if value in {"n", "nope"}:
+        return False
+    return default
+
+
+async def ratelimit(
+    redis: Redis[str],
+    redis_prefix: str,
+    remote_ip: str,
+    *,
+    bucket: None | str,
+    max_burst: int,
+    count_per_period: int,
+    period: int,
+    tokens: int,
+) -> tuple[bool, dict[str, str]]:
+    """Take b1nzy to space using Redis."""
+    remote_ip = hash_bytes(remote_ip.encode("ascii"))
+    key = f"{redis_prefix}:ratelimit:{remote_ip}"
+    if bucket:
+        key = f"{key}:{bucket}"
+
+    result = await redis.execute_command(
+        # type: ignore[no-untyped-call]
+        "CL.THROTTLE",
+        key,
+        max_burst,
+        count_per_period,
+        period,
+        tokens,
+    )
+
+    headers: dict[str, str] = {}
+
+    # fmt: off
+    # pylint: disable=line-too-long
+    if result[0]:
+        retry_after = result[3] + 1  # TODO: remove after brandur/redis-cell#58 is merged and a new release was made  # noqa: B950
+        headers["Retry-After"] = str(retry_after)
+        if not bucket:
+            headers["X-RateLimit-Global"] = "true"
+
+    if bucket:
+        assert bucket is not None
+        reset_after = result[4] + 1  # TODO: remove after brandur/redis-cell#58 is merged and a new release was made  # noqa: B950
+        headers["X-RateLimit-Limit"] = str(result[1])
+        headers["X-RateLimit-Remaining"] = str(result[2])
+        headers["X-RateLimit-Reset"] = str(time.time() + reset_after)
+        headers["X-RateLimit-Reset-After"] = str(reset_after)
+        headers["X-RateLimit-Bucket"] = hash_bytes(bucket.encode("ascii"))
+    # fmt: on
+
+    return bool(result[0]), headers
+
+
+def remove_suffix_ignore_case(string: str, suffix: str) -> str:
+    """Remove a suffix without caring about the case."""
+    if string.lower().endswith(suffix.lower()):
+        return string[: -len(suffix)]
+    return string
+
+
 def replace_umlauts(string: str) -> str:
     """Replace Ä, Ö, Ü, ẞ, ä, ö, ü, ß in string."""
     if string.isupper():
@@ -462,16 +564,6 @@ def replace_umlauts(string: str) -> str:
     )
 
 
-def emoji2code(emoji: str) -> str:
-    """Convert an emoji to the hexcodes of it."""
-    return "-".join(f"{ord(c):x}" for c in emoji).upper()
-
-
-def create_emoji_html(emoji: str, emoji_url: str) -> str:
-    """Create an HTML element that can be used to display an emoji."""
-    return f'<img src="{emoji_url}" alt="{emoji}" class="emoji">'
-
-
 async def run(
     program: str,
     *args: str,
@@ -499,14 +591,39 @@ def str_to_bool(val: None | str | bool, default: None | bool = None) -> bool:
         return val
     if isinstance(val, str):
         val = val.lower()
-        if val in {"1", "on", "s", "sure", "t", "true", "y", "yes"}:
+        if val in {
+            "1",
+            "a",
+            "accept",
+            "e",
+            "enabled",
+            "on",
+            "s",
+            "sure",
+            "t",
+            "true",
+            "y",
+            "yes",
+        }:
             return True
-        if val in {"0", "f", "false", "n", "no", "nope", "off"}:
+        if val in {
+            "0",
+            "d",
+            "disabled",
+            "f",
+            "false",
+            "n",
+            "no",
+            "nope",
+            "off",
+            "r",
+            "reject",
+        }:
             return False
         if val in {"idc", "maybe", "random"}:
-            return bool(random.randrange(2))
+            return bool(random.randrange(2))  # nosec: B311
     if default is None:
-        raise ValueError(f"Invalid truth value: {val}")
+        raise ValueError(f"invalid truth value {val!r}")
     return default
 
 
